// JUMA-TRX2 serial I/O test command set
// Juha Niinikoski, OH2NLT

// Serial test commands moved to own module 03.01.2007

// PA RF attenuator test 23.04.2007
// EEPROM Dump limits increased 08.03.2008
// Bar Graph test 'B' changed to 'b', better PA100 protocol compatibility  15.11.2008

#include <stdlib.h>
#include <stdio.h>					// Added - 5B4AIY
#include <ctype.h>
#include "juma-trx2.h"				// Board definitions
#include "DataEEPROM.h"				// EEPROM I/O definitions, interface to assembler module
#include "trx2_eeprom.h"			// EEPROM structure definitions

// External References
// External Functions
extern void putch(char);						// UART1 I/O
extern int kbhit(void);							// Check for characters in buffer
extern unsigned char getch(void);				// Get a character from the buffer
extern void ClearUSART1queue();					// Flush UART1 buffer
extern void putch2(char);						// UART2 I/O
extern int kbhit2(void);
extern unsigned char getch2(void);
extern void draw_s_meter(int);					// LCD functions
extern void lcd_putst(register const char *);
extern void display_screen(const char *, const char *);
extern void lcd_cmd(unsigned char);
extern void lcdoutch(unsigned char);			// Added - 5B4AIY
extern void set_chgen(int);
extern int convert_adc12(unsigned int);			// ADC
extern void exchg_data_spi1(unsigned int);
extern int set_scale(void);
extern void ms_delay(unsigned int);				// General delay
extern void us_delay(unsigned int);
extern int auto_equalise(int);
extern int cw_speed(int);
extern void display_hdr(void);
extern int read_defval(void);
extern void save_defval(void);
extern int read_calval(void);
extern void save_calval(int);
extern int RIT_freq(int);
extern int check_bit(int);
extern void max_min(int *, int, int);
extern void send_char(unsigned char);
extern void morse_practice(int);

// External Data
extern unsigned char morse_symbols[];
extern unsigned char lcdpbuff[];				// LCD print buffer, longer than the display for prints
extern int br_txt[];
extern char *s_meter_type[];
extern char *on_off[];
extern char *mode_msg[];
extern char *agc_speed_msg[];
extern char *audio_input[];
extern char *cw_spd_pot[];
extern char *band_limits[];
extern char *keyer_msg[];
extern char *power_msg[];
extern char *disp_type[];
extern char *cw_rls_speed[];
extern char *Not_Installed[];
extern char *start_page_select[];
extern char *button_type[];
extern char *pwr_mtr_msg[];
extern char *splash_type[];
extern char *mv[];
extern char *cw_annunciation_msg[];
extern char New_Line[];
extern unsigned char band[];
extern char Equalisation_Aborted[];

extern long freq_in_use;
extern long offset;
extern int isd_1700_installed;
extern int s_meter;
extern int cw_pot;
extern int cw_period;
extern int rit;
extern int rf_filt_idx;				// RF filter & attenuator switch table index
extern unsigned int hc595;			// Main & PA board SPI bus bits
extern int vfo_idx;					// VFO index, A = 0, B = 1, if (Split) then RX = 0, TX = 1
extern int cw_active;				// Status flag for keyer module
extern int PTT_IN;					// Atomic copy of PTT_IN_IO input / TX on, 0 = Active
extern int mute;					// Mute flag
extern int alarm_flag;
extern int loop_ctr;
extern unsigned int ms_counter;
extern int dot_time;
extern int morse_mode[];
extern int split_flag;
extern unsigned char morse_filter[];

// User Configuration Values
extern struct
	{
	struct defval defval;
	} eeprom;

// System Calibration Values
extern struct
	{
	struct calval calval;
	} cal;

// Synthesiser Register Values
extern union
	{
	struct DDS_Registers Registers;
	unsigned char AD9850[8];
	} DDS;

// Local Data
const char *bandswitch[] = {
							"Default",
							"User (Mode A)",
							"User (Mode B)",
							"Locked"
							};

const char *freq_display_type[] = {
							"Original",
							"New",
							"Fixed A",
							"Fixed B"
							};

const char *freq_display[] = {
							"  [12.345.68]\n\r  [ 2.345.68]\n\r  [ 0.345.68]\n\r  [ 0.045.68]\n\r  [ 0.005.68]\n\r  [ 0.000.68]\n\r  [ 0.000.08]",
							"  [12.345678]\n\r  [ 2.345678]\n\r  [  345.678]\n\r  [   45.678]\n\r  [    5.678]\n\r  [    0.678]\n\r  [    0.078]\n\r  [    0.008]",
							"  [12.345678]\n\r  [ 2.345678]\n\r  [ 0.345678]\n\r  [ 0.045678]\n\r  [ 0.005678]\n\r  [ 0.000678]\n\r  [ 0.000078]\n\r  [ 0.000008]",
							"  [12.34568 ]\n\r  [ 2.34568 ]\n\r  [ 0.34568 ]\n\r  [ 0.04568 ]\n\r  [ 0.00568 ]\n\r  [ 0.00068 ]\n\r  [ 0.00008 ]"
							};

const char dividing_line[] = {"------------------------------------------\n\r"};

const int filter_freq[] = {
							300,	// 0
							600,	// 1
							1,		// 2
							2,		// 3
							4,		// 4
							8,		// 5
							12,		// 6
							15,		// 7
							19,		// 8
							23,		// 9
							26,		// 10
							28		// 11
							};

const char *vfo_rounding[] = {
							"Off",
							"10Hz",
							"100Hz/10Hz",
							"1kHz/100Hz/10Hz"
							};

const char *two_band_filter[] = {
							"3.5MHz",
							"7.0MHz"
							};

const char *active_filter[] = {
							"Wide",
							"Narrow"
							};

const char *sb_select[] = {
							"USB",
							"LSB"
							};

const char *tx_mode[] = {
							"CW/Tune",
							"SSB"
							};

const char *auto_equalise_msg[] = {
						"Equalisation Complete!",		// 0 - Normal return
						"Power Measurement Error!",		// 1 - Abnormally low output
						"User Initiated Abort!",		// 2 - PTT switch prematurely released
						"Fault Detected!"				// 3 - SWR/Low Voltage/High Voltage/Over-current fault
						};

const char _lock[] = {
						SPACE,
						'*'
						};

const char *filters[] = {
						"Narrow",
						"Wide",
						"Transmit"
						};

const char *VFO_Equalisation[] = {
						"Off",
						"Split",
						"Split+Simplex"
						};

#if INCLUDE_TEST_CODE
const char *button_name[] = {
						"MODE",				// 0 - ASCII '1'
						"FILTER",			// 1 - ASCII '2'
						"RIT",				// 2 - ASCII '3'
						"VFO/A=B",			// 3 - ASCII '4'
						"FAST/VFAST",		// 4 - ASCII '5'
						"DISPLAY/CONFIG",	// 5 - ASCII '6'
						"PWR"				// 6 - ASCII '0'
						};
#endif

const char lcd_test[] = {"LCD Bar Graph & Character Test,"};
const char any_key_exits[] = {" any key exits."};
const char esc_exits[] = {" ESC exits."};
const char uart1_and_uart2_test[] = {"UART1/UART2 Character Cross-Copy Test,"};
const char write_ASCII[] = {"Write ASCII to LCD,"};
const char trip_fmt[] = {"%4i = %5.2f%c%s"};
const char PA_attenuator[] = {" PA Attenuator: %2idB%s"};
const char current_atten_setting[] = {"Current RF Attenuator Setting"};
const char V_A_fmt[] = {"%11.2f%c"};
const char Fwd_Rev_Pwr_fmt[] = {"%3s:%7.2fW"};
const char Seconds[] = {" Seconds"};
const char RIT_fmt[] = {"RIT: %+5iHz"};
const char All_Off[] = {"0 - All Alarms OFF"};
const char OC_Alarm[] = {"1 - Over-Current Alarm"};
const char SWR_Alarm[] = {"2 - High SWR Alarm"};
const char OV_Alarm[] = {"3 - Over-Voltage Alarm"};
const char LO_Batt_Alarm[] = {"4 - Low Battery (Under-Voltage Pre-Limit) Alarm"};
const char UV_Alarm[] = {"5 - Under-Voltage Alarm"};
const char All_On[] = {"6 - All Alarms ON"};
const char Disabled[] = {" is disabled."};
const char Two_Strings[] = {"%s%s"};
const char Four_Strings[] = {"%s%s%s%s"};
const char Test_Terminated[] = {"Test Terminated.\n\r"};

int ascii_freq(long freq)
	{
	int c, i, scale = 'M', dp = 2;
	long divisor = 10000000L;

	if(freq < 1000000L)
		{
		scale = 'k';
		dp = 5;
		}

	for(i = 0; i < 9; i++, divisor /= 10L)
		{
		c = (int)(freq / divisor);					// Extract next digit
		freq -= (long)c * divisor;					// Remove current decade value

		if((c == 0) && (i < (dp - 1))) c = SPACE;	// Suppress leading zeros,
		else c += ZERO;								// or convert to ASCII

		if(i == dp) lcdpbuff[i++] = '.';			// Insert decimal point, and increment the buffer index,

		lcdpbuff[i] = (char)c;						// and insert the next digit in the buffer.
		}

	lcdpbuff[9] = 0;								// Terminate the buffer.
	return scale;
	}

#if INCLUDE_TEST_CODE
void print_time(double time)
	{
	if(time) printf("Contact Closure Time:%9.1f mSec%s", time / 6.73423, New_Line);
	}

void increment_count(double *c)
	{
	(*c)++;
	us_delay(100);
	}
#endif

void print_off(void)
	{
	printf(Two_Strings, on_off[0], New_Line);
	}

void print_filter_freq(int index, int select)
	{
	printf((select) ? "%3i%cHz" : "%i%cHz", filter_freq[index], (index < 2) ? 'k' : 'M');
	}

void dump_eeprom(void)
	{
	int i;
	unsigned int w;

	printf("%sDump EEPROM contents%s%5s%20s%s%.23s%.22s", New_Line, New_Line, "ADDR", "DATA", New_Line, dividing_line, dividing_line);

	for(i = 0; i < 512; i += 2)								// Byte address for EEPROM, but data is stored in words
		{
		ReadEE(EEPAGE, (i + EEDEF), (int *)&w, WORD);		// EEPROM Address 8 high bits, address + physical EEPROM start 16 low bits - Added cast - 5B4AIY

		if((i % 16) == 0) printf("%s%5.4X", New_Line, i);

		printf("%5.4X", w);
		}
	printf("%s%.23s%.22s%s", New_Line, dividing_line, dividing_line, New_Line);
	}

void clear_counter(void)
	{
	int data = 0;

	EraseEE(EEPAGE, EE_FD_LOC, WORD);
	WriteEE(&data, EEPAGE, EE_FD_LOC, WORD);
	dump_eeprom();
	}
	
double get_factor(double adc, int cal_factor, double divisor)
	{
	return adc * (double)cal_factor / divisor;
	}

void Display_Attenuator_Setting(int direction)
	{
	int atten;

	atten = (int)cal.calval.rf_atn_bits[rf_filt_idx];
	atten += direction;
	max_min(&atten, 3, 0);
	cal.calval.rf_atn_bits[rf_filt_idx] = (unsigned char)atten;
	printf("Filter:");
	print_filter_freq(rf_filt_idx, TRUE);
	printf(PA_attenuator, atten, New_Line);
	}

void ad_msg0(double factor)
	{
	printf(V_A_fmt, get_factor(factor, cal.calval.id_mult, 2000000.0), 'A');
	}

void ad_msg1(double factor)
	{
	printf(V_A_fmt, get_factor(factor, cal.calval.batt_mult, 1000000.0), 'V');
	}

void ad_msg2(double factor)
	{
	int i;

	i = set_scale();
	printf((i > 9) ? "  S9+%2idB" : "     S%1i", i);
	}

void ad_msg3(double factor)
	{
	printf(Fwd_Rev_Pwr_fmt, "Rev", get_factor(factor * factor, cal.calval.fwd_pwr_mult, 1000000000.0));
	}

void ad_msg4(double factor)
	{
	printf(Fwd_Rev_Pwr_fmt, "Fwd", get_factor(factor * factor, cal.calval.fwd_pwr_mult, 1000000000.0));
	}

void ad_msg5(double factor)
	{
	printf("CW Speed Pot");
	}

void ad_msg6(double factor)
	{
	printf(RIT_fmt, RIT_freq((int)factor));
	}

void (*ad_msg[])(double) = {
						ad_msg0,
						ad_msg1,
						ad_msg2,
						ad_msg3,
						ad_msg4,
						ad_msg5,
						ad_msg6
						};

void alarm_msg0(void)		// All Alarms OFF
	{
	alarm_flag = OFF;
	printf(All_Off);
	}

void alarm_msg1(void)		// Over-Current Alarm
	{
	if(cal.calval.alarm_flags & OC_AL_ON)
		{
		alarm_flag |= OC_AL_ON;
		printf(OC_Alarm);
		}
	else printf(Two_Strings, OC_Alarm, Disabled);
	}

void alarm_msg2(void)		// High SWR Alarm
	{
	alarm_flag |= SWR_AL;
	printf(SWR_Alarm);
	}

void alarm_msg3(void)		// Over-Voltage Alarm
	{
	if(cal.calval.alarm_flags & HI_V_AL_ON)
		{
		alarm_flag |= HI_V_AL_ON;
		printf(OV_Alarm);
		}
	else printf(Two_Strings, OV_Alarm, Disabled);
	}

void alarm_msg4(void)		// Low-Voltage (Pre-Limit Alarm)
	{
	if(cal.calval.alarm_flags & LO_V_AL_ON)
		{
		alarm_flag |= BATT_LO;
		printf(LO_Batt_Alarm);
		}
	else printf(Two_Strings, LO_Batt_Alarm, Disabled);
	}

void alarm_msg5(void)		// Low-Voltage (Final Limit Alarm)
	{
	if(cal.calval.alarm_flags & LO_V_AL_ON)
		{
		alarm_flag |= LO_V_AL_ON;
		printf(UV_Alarm);
		}
	else printf(Two_Strings, UV_Alarm, Disabled);
	}

void alarm_msg6(void)		// All Alarms ON
	{
	alarm_flag = (OC_AL_ON | SWR_AL | HI_V_AL_ON | LO_V_AL_ON | BATT_LO);
	printf(All_On);
	}
/*
 The desired order for the alarms tests is:
 0 - Alarms OFF
 1 - Over-Current
 2 - SWR
 3 - Over-Voltage
 4 - Under-Voltage (Pre-Limit)
 5 - Under-Voltage (Final Limit)
 6 - All ON
 The character obtained from the buffer is subject to a %7 operation, and thus this resolves to:
 ASCII	DECIMAL	%7
   0       48    6
   1       49    0
   2       50    1
   3       51    2
   4       52    3
   5       53    4
   6       54    5
 Consequently the order of the functions has to be as shown below. A.Ryan - 22/OCT/2015
*/
void (*alarm_msg[])(void) = {
							alarm_msg1,	// 0 - Over-Current Alarm				(ASCII '1')
							alarm_msg2,	// 1 - High SWR Alarm					(ASCII '2')
							alarm_msg3,	// 2 - Over-Voltage Alarm				(ASCII '3')
							alarm_msg4,	// 3 - Under-Voltage Pre-Limit Alarm	(ASCII '4')
							alarm_msg5,	// 4 - Under-Voltage Final Limit Alarm	(ASCII '5')
							alarm_msg6,	// 5 - All Alarms ON					(ASCII '6')
							alarm_msg0	// 6 - All Alarms OFF					(ASCII '0')
							};

void serial_test(void)
	{
	#if INCLUDE_TEST_CODE
	double count;
	#endif
	int i, x, temp;
	unsigned int y = 0;
	unsigned char c;

	if(kbhit())
		{
		c = toupper(getch());

		switch(c)
			{
// Info commands
			case 'H':		// System info
			case '?':
				display_hdr();
				printf("\n\r%.21s[COMMAND TABLE]%.21s%s", dividing_line, dividing_line, New_Line);
				printf("H/? Help - (This Screen)%s", New_Line);
				printf("A   Display A-D Convertor Samples%s", New_Line);
				printf("B   Alarm Function Test%s", New_Line);
				printf("C   %s%s%s", lcd_test, any_key_exits, New_Line);
				printf("D   Clear Factory Default reset counter%s", New_Line);
				printf("E   Continuous SPI write,%s%s", any_key_exits, New_Line);
				printf("F   %s%s%s", uart1_and_uart2_test, esc_exits, New_Line);
				printf("G   Dump EEPROM contents%s", New_Line);
				printf("I   Frequency Display Examples%s", New_Line);
				printf("J   Display SPI bus control bits%s", New_Line);
				printf("K   Morse Character Test,%s%s", esc_exits, New_Line);
				printf("L   Reference Oscillator calibration value%s", New_Line);
				printf("M   Display scaled S-Meter & CW Speed Pot A-D value%s", New_Line);
				printf("N   Auto Equalise RF Output Power%s", New_Line);
				printf("O   RF Attenuator Settings%s", New_Line);
				printf("P   Toggle SCAF%s", New_Line);
				printf("Q   Single SPI write%s", New_Line);
				printf("R   Dump User Frequency Memory%s", New_Line);
				printf("S   Dump Transceiver Configuration Settings%s", New_Line);
				printf("T   %s%s%s", write_ASCII, esc_exits, New_Line);
#if INCLUDE_TEST_CODE
				printf("U   Test Push Buttons,%s%s", any_key_exits, New_Line);
				printf("V   Erase EEPROM%s", New_Line);
				printf("*   Dump Internal Values%s", New_Line);
#endif
				printf("+   Increment RF Attenuator%s", New_Line);
				printf("-   Decrement RF Attenuator%s", New_Line);
				printf("Z   Divide-By-Zero Trap%s", New_Line);
			break;

// Test commands
// ADC & PWM DAC test
			case 'A':		// ADC Test
				printf("%sA-D Convertor Samples%s Channel   Sample   Scaled(V)   Displayed%s%s", New_Line, New_Line, New_Line, dividing_line);
				x = 9;

				do	{
					temp = convert_adc12(x);
					printf("%5i%11i%11.3f  ", x, temp, get_factor((double)temp, 5000, 4096000.0));
					ad_msg[x - 9]((double)temp);
					printf(New_Line);
					} while (++x < 16);

				printf(dividing_line);
			break;

			case 'B':			// Alarm System test
				printf("%sSet Alarm Flag%sEnter 0,1,2,3,4,5 or 6 ...%s", New_Line, New_Line, New_Line);
				c = (getch() % 7);				// Ensure value is in the range 0 - 6
				alarm_msg[(int)c]();
				printf(New_Line);
				loop_ctr = 0;	// Reset alarm blink counter
			break;

			case 'C':			// Bar graph test
				temp = eeprom.defval.S_Meter_Type;
				eeprom.defval.S_Meter_Type = GRAPHIC;
				printf("%s%s%s%sSelect Scale: 0,1,2: ", New_Line, lcd_test, any_key_exits, New_Line);
				c = (getch() & 0x03) % 3;		// Ensure value is in the range 0 - 2
				printf("%d - %s%s", c, disp_type[c], New_Line);
				set_chgen(c);					// Select the desired character generator map, and clear the display.
				c = 0;							// Initialise the character

				do	{							// The maximum value for the S-meter is 48.
					x = 0;
					lcd_cmd(LINE2PLUS4);
					lcdoutch('[');
					lcdoutch(c);
					sprintf((char *)lcdpbuff, " =%3.2X]", c);	// Display LCD characters - Added cast - 5B4AIY
					lcd_putst((char *)lcdpbuff);

					do	{
						lcd_cmd(LINE1PLUS4);
						draw_s_meter(x);
						ms_delay(10);
						} while (++x < 49);

					c++;
					ms_delay(500);
					} while(!kbhit());

				ClearUSART1queue();						// Consume the break character by flushing the buffer
				eeprom.defval.S_Meter_Type = temp;		// Restore S-Meter type
				set_chgen(Display_Scale);				// Load the original bar graph fonts
			break;

			case 'D':		// Clear factory default reset counter
				printf("%sEEPROM Reset Counter cleared.", New_Line);
				clear_counter();
			break;

			case 'E':		// Continuous test write SPI bus
				printf("%sContinuous SPI Transmit,%s%s", New_Line, any_key_exits, New_Line);

				do	{
					y++;
					exchg_data_spi1(y);
					ms_delay(10);
					} while(!kbhit());

				ClearUSART1queue();			// Flush keyboard buffer
				printf(Test_Terminated);
			break;

			case 'F':		// Debug RS232 input, cross copy
				printf(Four_Strings, New_Line, uart1_and_uart2_test, esc_exits, New_Line);
				clear_lcd();

				do	{
					putch2(c = getch());
					lcd_cmd(LINE1);
					lcdoutch(c);

					if(kbhit2())
						{
						putch(c = getch2());
						lcd_cmd(LINE2);
						lcdoutch(c);
						}
					} while(c != 0x1B);	// Exit with ESC

				printf(Test_Terminated);
			break;

			case 'G':		// Dump EEPROM
				dump_eeprom();
			break;

			case 'I':
				printf("%sSelect Frequency Display Type: 0,1,2,3: ", New_Line);
				i = getch() % 4;		// Ensure value is in the range 0 - 3
				printf("%d - %s%s%.15s%s%s%s%.15s%s", i, freq_display_type[i], New_Line, dividing_line, New_Line, freq_display[i], New_Line, dividing_line, New_Line);
			break;

			case 'J':		// Print RF Attenuator setting & SPI bus bits
				printf("%sFully Decoded SPI Bus Data%s%.26s%s", New_Line, New_Line, dividing_line, New_Line);
				printf("Data            : 0x%4.4X%s", hc595, New_Line);
				printf("Attenuator      : %ddB%s", ((hc595 & 0xC000) >> 14), New_Line);
				printf("Spare Bits      : %d%s", ((hc595 & 0x3800) >> 11), New_Line);
				printf("Filter Bits     : %d%sFrequency       : ", ((hc595 & 0x0700) >> 8), New_Line);
				print_filter_freq(rf_filt_idx, FALSE);
				printf("%sDual-Band Filter: %s%s", New_Line, two_band_filter[((hc595 & 0x0080) >> 7)], New_Line);
				printf("Audio Source    : %s%s", audio_input[((hc595 & 0x0040) >> 6)], New_Line);
				printf("Noise Blanker   : %s%s", on_off[((hc595 & 0x0020) >> 5)], New_Line);
				printf("Speech Processor: %s%s", on_off[((hc595 & 0x0010) >> 4)], New_Line);
				printf("AGC Speed       : %s%s", agc_speed_msg[((hc595 & 0x0008) >> 3)], New_Line);
				printf("Active Filter   : %s%s", active_filter[((hc595 & 0x0004) >> 2)], New_Line);
				printf("Sideband        : %s%s", sb_select[((hc595 & 0x0002) >> 1)], New_Line);
				printf("Mode            : %s%s%.26s%s", tx_mode[(hc595 & 0x0001)], New_Line, dividing_line, New_Line);
			break;

			case 'K':		// Send Morse text test
				printf("%sMorse Character Test%s", New_Line, New_Line);

				do	{
					c = toupper(getch());
					dot_time = cw_speed(convert_adc12(CW_POT));	// Get new speed

					switch(c)
						{
						case '0' ... '9':
							printf("%c", c);
							send_char(morse_symbols[c - '0']);	// Convert character to 0 - 9 index into Morse character array
						break;

						case 'A' ... 'Z':
							printf("%c", c);
							send_char(morse_symbols[c - 55]);	// Convert A - Z into index into Morse character array. (Letters start at 10)
						break;

						default:
							if(c == 0x0D || c == 0x0A) printf(New_Line);
							else printf(" ");

							ms_delay(dot_time * 4);		// Send word space (7 dot-times, 3 dot-times already sent at the end of the character)
						break;
						}
					} while (c != 0x1B);

				dot_time = eeprom.defval.cw_annunciation >> 8;	// Reset the dot time for CW annunciation
				printf(Two_Strings, New_Line, Test_Terminated);
			break;

			case 'L':		// Test, print ref osc calibration value
				printf("%sRef Osc = %li MHz%s", New_Line, cal.calval.ref_osc, New_Line);
			break;

			case 'M':		// S-meter, CW/Squelch pot, equivalent CW speed in wpm
				printf("%sS-Meter = %2i POT = %4i WPM = %2i%s", New_Line, s_meter, cw_pot, 1200 / cw_speed(cw_pot), New_Line);  // Fixed bug - 5B4AIY
			break;

			case 'N':		// Auto-Equalise RF O/P power
				printf("%sAuto Equalise RF Output Power%s", New_Line, New_Line);
				printf("Ensure transceiver is connected to a 50 Ohm dummy load.%s", New_Line);
				printf("This may alter your current attenuator settings.%s", New_Line);
				printf("Do you wish to continue (Y/N)?: ");
				putch(c = toupper(getch()));
				printf(New_Line);
				x = ABORT;

				if(c == 'Y')
					{
					x = auto_equalise(TRUE) & 0x00FF;		// Mask off the reference band data
					printf(Two_Strings, auto_equalise_msg[x], New_Line);
					}

				if(x) printf("Equalisation Aborted - No changes made.%s", New_Line);
			break;

			case 'O':		// Display RF Attenuator Settings
				printf("%sRF Attenuator Settings%s Filter  Attenuator%s%.20s%s", New_Line, New_Line, New_Line, dividing_line, New_Line);
				i = 11;

				do	{
					printf(" ");
					print_filter_freq(i, TRUE);
					printf("%7idB%s", cal.calval.rf_atn_bits[i], New_Line);
					} while (i--);

				printf("%.20s%s", dividing_line, New_Line);
			break;

			case 'P':		// Toggle SCAF
				printf("%sAudio %s%s", New_Line, on_off[mute], New_Line);
				mute ^= 0x0001;
			break;

			case 'Q':		// Test write SPI bus
				printf("%sSPI Tx%s", New_Line, New_Line);
				exchg_data_spi1(0x5501);
			break;

			case 'R':		// Display VFO Memory Settings
				printf("%s    VFO Memory Settings%s%.27s%s", New_Line, New_Line, dividing_line, New_Line);

				for(i = 0; i < 26; i++)
					{
					c = (char)ascii_freq(eeprom.defval.vfo[i]);
					printf("%s-%c: %s %cHz %c%5s%s", mv[(i < maximum_VFO)], (char)(i + 'A'), lcdpbuff, c, (char)_lock[check_bit(i)], mode_msg[eeprom.defval.mode[i]], New_Line);
					}

				printf("%s User Frequency Settings%s%.27s%s", New_Line, New_Line, dividing_line, New_Line);

				for(i = 30; i < 39; i++)
					{
					c = (char)ascii_freq(eeprom.defval.vfo[i]);
					printf("%4im: %s %cHz%7s%s", band[i - 29], lcdpbuff, c, mode_msg[eeprom.defval.mode[i]], New_Line);
					}

				printf("%.27s%s", dividing_line, New_Line);
			break;

			case 'S':		// Dump Transceiver configuration settings
// ************* System Settings, Firmware Version & Build *************
				printf("%s%31s%s", New_Line, "SYSTEM CALIBRATION SETTINGS", New_Line);
				printf("     Firmware: %s Build: %2s%s%.36s%s", VERSION, BUILD, New_Line, dividing_line, New_Line);
				printf("Reference Oscillator: %9li Hz%s", cal.calval.ref_osc, New_Line);
				printf("DC Voltmeter        : %4d%s", cal.calval.batt_mult, New_Line);
				printf("Ammeter             : %4d%s", cal.calval.id_mult, New_Line);
				printf("Power Meter         : %4d%s", cal.calval.fwd_pwr_mult, New_Line);
				printf("S-Meter             : %-d%s", cal.calval.s_mtr_cal, New_Line);

				if(cal.calval.beep_len) printf("Beep Time           : %-d mSec%s", cal.calval.beep_len, New_Line);
				else printf("Beep                : OFF%s", New_Line);

				printf("SWR Trip Setting    : %-.2f:1%s", ((double)cal.calval.swr_trip) / 100.0, New_Line);
				printf("SWR Long Delay Time :");

				if(cal.calval.swr_delay) printf(" %-.1f Secs%s", (double)cal.calval.swr_delay, New_Line);
				else printf(" Disabled%s", New_Line);

				printf("Over-Current Trip   : ");
				if(cal.calval.alarm_flags & OC_AL_ON)
					printf(trip_fmt, cal.calval.overcurrent_trip, get_factor((double)cal.calval.overcurrent_trip, cal.calval.id_mult, 2000000.0), 'A', New_Line);
				else print_off();

				printf("Over-Voltage Trip   : ");
				if(cal.calval.alarm_flags & HI_V_AL_ON)
					printf(trip_fmt, Over_Voltage, get_factor((double)Over_Voltage, cal.calval.batt_mult, 1000000.0), 'V', New_Line);
				else print_off();

				printf("Under-Voltage Trip  : ");
				if(cal.calval.alarm_flags & LO_V_AL_ON)
					printf(trip_fmt, Under_Voltage, get_factor((double)Under_Voltage, cal.calval.batt_mult, 1000000.0), 'V', New_Line);
				else print_off();

				printf("Pre-Limit Trip      : ");
				if(cal.calval.alarm_flags & LO_V_AL_ON)
					printf(trip_fmt, cal.calval.pre_limit_trip, get_factor((double)cal.calval.pre_limit_trip, cal.calval.batt_mult, 1000000.0), 'V', New_Line);
				else print_off();

				printf("Long Button Mode    : %s%s", button_type[hold_latch], New_Line);
				printf("Band Limits         :%s%s", band_limits[cal.calval.Band_Limits], New_Line);
				printf("Splash Screen       : %s%s", splash_type[cal.calval.splash], New_Line);

// ************* User Configuration Settings *************
				printf("%s%31s%s%.36s%s", New_Line, "USER CONFIGURATION SETTINGS", New_Line, dividing_line, New_Line);
				printf("AGC Speed           : %s%s", agc_speed_msg[AGC_speed], New_Line);

				i = 0;

				do {			// Filter Cut-Off Frequencies
					printf("Filter BW %-10s: %4li Hz%s", filters[i], 75000L / eeprom.defval.scaf_filters[i], New_Line);
					} while (++i < 3);

				printf("Audio Source        : %s%s", audio_input[AF_Input], New_Line);
				printf("Speech Processor    : %s%s", on_off[Speech_Processor], New_Line);
				printf("Keyer Mode          : %8s%s", keyer_msg[Keyer_Mode], New_Line);
				printf("CW Release Delay    : ");

				if(eeprom.defval.CW_Release > MIN_CW_RELEASE) printf("%-d mSec", eeprom.defval.CW_Release);
				else printf(cw_rls_speed[eeprom.defval.sq]);

				printf("\n\rCW Pitch            : %-d Hz%s", eeprom.defval.cw_shift, New_Line);
				printf("CW Pot Mode         : %s%s", cw_spd_pot[eeprom.defval.sq], New_Line);
				temp = cw_speed(Keyer_Speed & UNLOCKED);
				printf("Keyer Speed         : ");

				if(Keyer_Speed & LOCKED) printf("Locked%5d wpm%s", (1200 / temp), New_Line);
				else printf("Unlocked%3d wpm%s", (1200 / temp), New_Line);

				printf("CW/CWR Swap         : %s%s", on_off[eeprom.defval.CW_CWR_Swap], New_Line);
				printf("LCD Backlight       : %-d%s", eeprom.defval.back_light, New_Line);
				printf("LCD Contrast        : %-d%s", eeprom.defval.contrast, New_Line);
				printf("LCD Timer           : %-d%s", eeprom.defval.back_light_timer * 5, Seconds);

				if(!eeprom.defval.back_light_timer) printf(" (Off)");

				printf("%sRS-232 Mode         : Test%s", New_Line, New_Line);
				printf("Speed               : %-d00 Baud%s", br_txt[Baud_Rate], New_Line);
				printf("Active VFO Mode     : %2i, %s", maximum_VFO, (maximum_VFO == SPLIT_MODE) ? "A/B Split" : "Multi-Memory");
				printf("%sPassive VFO (Memory): %2i", New_Line, 26 -  maximum_VFO);
				printf("%sVFO Equalisation    : %s", New_Line, VFO_Equalisation[eeprom.defval.VFO_Equalisation]);
				printf("%sS-Meter             : %7s%s", New_Line, s_meter_type[eeprom.defval.S_Meter_Type], New_Line);
				printf("S-Meter Hold Time   : %-.2f%s%s", ((double)eeprom.defval.Hold_Time) / 1000.0, Seconds, New_Line);
				printf("TX Disable          : %s%s", on_off[eeprom.defval.OOB_Check], New_Line);
				printf("Auto-Sideband       : %s%s", on_off[eeprom.defval.Auto_SB], New_Line);
				printf("Frequency Display   : %s%s", freq_display_type[eeprom.defval.Freq_Display], New_Line);
				printf("Rapid Band-Switch   : %s%s", bandswitch[RBS_Mode], New_Line);
				printf("Include RIT         : %s%s", on_off[eeprom.defval.RIT_Include], New_Line);
				printf("Tune Attenuator     :%3idB%s", -eeprom.defval.Tune_Attenuation, New_Line);
				printf("VFO Adjustment      : ");

				if(eeprom.defval.VFO_Adjust)
					printf("%s decade(s)%s", vfo_rounding[eeprom.defval.VFO_Adjust], New_Line);
				else print_off();

				printf("Meter Scale Type    : %s%s", disp_type[Display_Scale], New_Line);
				printf("Power-Off Mode      : %8s%s", power_msg[eeprom.defval.nb], New_Line);
				printf("Power Meter Type    : %s%s", pwr_mtr_msg[eeprom.defval.Power_Meter], New_Line);
				printf("Start-Up Page       : %s", start_page_select[display_page & 0x07]);

				if((display_page & 0x07) == LAST_PAGE)
					printf(" - %s", start_page_select[display_page >> 4]);

				printf("%sCW Annunciation     : ", New_Line);

				if(eeprom.defval.cw_annunciation)
					printf("%s at %2dWPM%s", cw_annunciation_msg[eeprom.defval.cw_annunciation & 0x00FF], 1200 / (eeprom.defval.cw_annunciation >> 8), New_Line);
				else print_off();

				printf("Transmit Delay      : %-d mSec%s", eeprom.defval.TX_Delay, New_Line);
				printf("Voice Memory        : %sInstalled%s", Not_Installed[isd_1700_installed], New_Line);
				printf("%.36s%s", dividing_line, New_Line);
			break;

			case 'T':		// Test LCD character generator, ASCII
				printf(Four_Strings, New_Line, write_ASCII, esc_exits, New_Line);
				clear_lcd();
				x = 0;

				do	{
					c = getch();

					if(x == 16) lcd_cmd(LINE2);
					if(c == 0x0D || x == 32)
						{
						clear_lcd();
						x = 0;
						}

					if(isprint(c))
						{
						printf("Char = 0x%.2X\n\r", c);
						lcdoutch(c);
						x++;
						}
					else printf("Char = 0x%.2X - Non-Printable Character\n\r", c);
					} while(c != 0x1B);

				printf(Test_Terminated);
			break;

#if INCLUDE_TEST_CODE
			case 'U':		// Test code to measure the shortest contact closure time.
				printf("%sButton Test%s", New_Line, New_Line);
				printf("Select Button%sEnter 0,1,2,3,4,5 or 6 ...%s", New_Line, New_Line);
				c = (getch() % 7);				// Ensure value is in the range 0 - 6
				printf("Button: %s ...%s", button_name[c], New_Line);

				switch(c)
					{
					case 0:			// MODE button (ASCII: '1')
						do {
							count = 0;

							while(_MODE) increment_count(&count);

							print_time(count);
							} while (!kbhit());
					break;

					case 1:			// FILTER button (ASCII: '2')
						do {
							count = 0;

							while(_FILTER) increment_count(&count);

							print_time(count);
							} while (!kbhit());
					break;

					case 2:			// RIT button (ASCII: '3')
						do {
							count = 0;

							while(_RIT) increment_count(&count);

							print_time(count);
							} while (!kbhit());
					break;

					case 3:			// VFO/A=B button (ASCII: '4')
						do {
							count = 0;

							while(_VFO) increment_count(&count);

							print_time(count);
							} while (!kbhit());					
					break;

					case 4:			// FAST/VFAST button (ASCII: '5')
						do {
							count = 0;

							while(_VFAST) increment_count(&count);

							print_time(count);
							} while (!kbhit());					
					break;

					case 5:			// DISPLAY button (ASCII: '6')
						do {
							count = 0;

							while(_DISPLAY) increment_count(&count);

							print_time(count);
							} while (!kbhit());					
					break;

					case 6:			// PWR button (ASCII: '0')
						do {
							count = 0;

							while(PWR_SW) increment_count(&count);

							print_time(count);
							} while (!kbhit());					
					break;
					}
				ClearUSART1queue();	// Flush keyboard buffer
				printf(Test_Terminated);
			break;

			case 'V':		// Only used for Test & Debug
				dump_eeprom();
				printf("\n\rErasing EEPROM...");

				for(i = 0; i < 512; i += 2) EraseEE(EEPAGE, (i + EEDEF), WORD);

				clear_counter();
				printf("Saving existing calibration and configuration...");
				save_calval(1);
				save_defval();
				dump_eeprom();
				printf("\n\rReading EEPROM...");
				if(read_calval() || read_defval()) printf("\n\rCHECKSUM ERROR!"); else printf("\n\rChecksum OK");
				printf("\n\r");
			break;

			case '*':		// Test, debug print some internal variables
				printf("%sSpecial Internal Variable Dump", New_Line);
				printf("%seeprom.defval.max_vfo =%3i,%15s", New_Line, maximum_VFO, (maximum_VFO == SPLIT_MODE) ? "A/B+Split" : "Multi-VFO");
				printf("%seeprom.defval.vfo_sel =%3i,", New_Line, Select_VFO);

				if(split_flag) printf("%23s", "Split Mode Active");
				else printf("%10s%c", "VFO-", (char)Select_VFO + 'A');

				printf("%svfo_idx =%3i,%24s%c", New_Line, vfo_idx, "VFO-", (char)vfo_idx + 'A');
				printf("%seeprom.defval.mode[vfo_idx] = %i,%5s", New_Line, eeprom.defval.mode[vfo_idx], mode_msg[eeprom.defval.mode[vfo_idx]]);
				printf("%seeprom.defval.mode[2] = %i,%11s", New_Line, eeprom.defval.mode[2], mode_msg[eeprom.defval.mode[2]]);
				printf("%sPTT_IN = %i,%30s", New_Line, PTT_IN, (PTT_IN) ? "Receive " : "Transmit");
				printf("%scw_active = %i", New_Line, cw_active);
				printf("%sfreq_in_use  :%11ld", New_Line, freq_in_use);
				printf("%sDDS Frequency:%11ld", New_Line, (freq_in_use + offset));
				printf("%sOffset       :%+11ld", New_Line, offset);
				printf("%sRIT          :%+11d", New_Line, rit);
				printf("%sRef Frequency:%11ld", New_Line, cal.calval.ref_osc);
				printf("%sDDS Number   :%11lld", New_Line, DDS.Registers.Frequency);
				i = 0;

				do	{
					printf("%sAD9851[%1d]:%3.2X", New_Line, i, DDS.AD9850[i]);
					} while (++i < 4);

				printf("%smSec Counter = %i%s", New_Line, ms_counter, New_Line);
			break;
#endif
			case '+':		// Increment Current Band Attenuator
				printf("%sIncrement %s%s", New_Line, current_atten_setting, New_Line);
				Display_Attenuator_Setting(PLUS_ONE);
			break;

			case '-':		// Decrement Current Band Attenuator
				printf("%sDecrement %s%s", New_Line, current_atten_setting, New_Line);
				Display_Attenuator_Setting(MINUS_ONE);
			break;

			case 'Z':		// Trap / Error test
				printf("%sDivide By Zero%s", New_Line, New_Line);
				x = 0;
				temp = temp / x;			// Generate error ==> trap
			break;

			default:
				printf("%sInvalid Command: ", New_Line);

				if(isprint(c)) printf("%c", c);
				else printf("(Non-Printable Character)");

				printf(" Value: %d%s", c, New_Line);
				ClearUSART1queue();			// Flush the buffer.
			break;
			}	// case
		}	// if
	}

